#include <ArduinoBLE.h>

// ********************************************************************************************************
#define uS_TO_S_FACTOR  1000000ULL    // Conversion factor for micro seconds to seconds
#define TIME_TO_SLEEP   10            // Time ESP32 will go to sleep (in seconds)
#define LED             21            // user LED pin
// ********************************************************************************************************

BLEService bleServiceTest("776982a9-8315-4bad-80f7-325e6a521aa6");
BLECharacteristic bleCharacteristicSwitch("df172997-36a6-446c-b70c-ac1556012427", BLERead | BLEWrite, 32);

void bleOnConnected(BLEDevice bleDevice)
{
  Serial.println("bleOnConnected: bleDevice.address()=" + bleDevice.address());
  Serial.println("BLE.stopAdvertise()");
  BLE.stopAdvertise();  //no effect on XIAO_ESP32S3
}

void bleOnDisconnected(BLEDevice bleDevice)
{
  Serial.println("bleOnDisconnected: bleDevice.address()=" + bleDevice.address());
  Serial.println("BLE.stopAdvertise()");
  BLE.stopAdvertise();  //no effect on XIAO_ESP32S3
  BLE.end(); //crashes on XIAO_ESP32S3
// ************************************************************************************************
  Serial.println("BLE END");
  for(int i = 0; i < 10; i++) {
    digitalWrite(LED, LOW);
    delay(100);
    digitalWrite(LED, HIGH);
    delay(100);      
  }  
  esp_deep_sleep_start();
// ************************************************************************************************
}

void bleOnCharacteristicStringWritten(BLEDevice bleDevice, BLECharacteristic characteristic)
{
  String sWritten = String (((char*)characteristic.value()), characteristic.valueLength());
  Serial.println("bleOnCharacteristicStringWritten: " + sWritten);
}

void bleOnCharacteristicStringRead(BLEDevice bleDevice, BLECharacteristic characteristic)
{
  characteristic.setValue(String("<123454321>").c_str());
  String sRead = String (((char*)characteristic.value()), characteristic.valueLength());
  Serial.println("bleOnCharacteristicStringRead: " + sRead);
}

void setup() 
{
  Serial.begin(115200);
  while (!Serial) 
    delay(10);
// ****************************************************************************************
  pinMode(LED, OUTPUT);
  for(int i = 0; i < 5; i++) {
    digitalWrite(LED, LOW);
    delay(100);
    digitalWrite(LED, HIGH);
    delay(100);      
  }   
  esp_sleep_enable_timer_wakeup(TIME_TO_SLEEP * uS_TO_S_FACTOR);
// ****************************************************************************************

  Serial.println("setup: calling BLE.begin()");
  if (!BLE.begin())
    Serial.println("BLE.begin() failed");
  BLE.setLocalName("BLE Server Test");
  BLE.setAdvertisedService(bleServiceTest);
  bleServiceTest.addCharacteristic(bleCharacteristicSwitch);
  BLE.addService(bleServiceTest);
  BLE.setEventHandler(BLEConnected, bleOnConnected);
  BLE.setEventHandler(BLEDisconnected, bleOnDisconnected);
  
  bleCharacteristicSwitch.setEventHandler(BLEWritten, bleOnCharacteristicStringWritten);
  bleCharacteristicSwitch.setEventHandler(BLERead, bleOnCharacteristicStringRead);
  bleCharacteristicSwitch.setValue(String("INIT").c_str());
  
  BLE.advertise();
}

void loop() 
{
  static uint32_t uLoopCount = 0;
  uLoopCount++;
  
  if (uLoopCount == 1)
    Serial.println("loop: first call");

  BLE.poll();

  delay(10);
}